#pragma once

/* #include <gz-util/log.hpp> */

#include <initializer_list>

#include <list>
#include <map>
#include <cstdint>
#include <string>


namespace gz::vlk {
	/**
	 * @brief Information on a single block of memory
	 */
	struct Block {
		size_t size = 0;
		size_t offset = 0;
		bool free = true;
		std::string toString() const;
	};

	/**
	 * @brief General purpose dynamic sized block allocator
	 * @note Does not actually allocate anything itself
	 */
	class DynBlockAllocator {
		public:
			DynBlockAllocator(uint32_t size);
			/**
			 * @brief Allocate block of size with alignment
			 * @throws VkException if the block can not be allocated
			 */
			[[ nodiscard ]] uint32_t allocate(uint32_t size, uint32_t alignment);
			/**
			 * @brief Free a block allocated with allocate()
			 */
			void free(uint32_t offset);

			using Iterator = std::list<Block>::iterator;
			const Iterator begin() { return blocks.begin(); }
			const Iterator end() { return blocks.end(); }
			std::list<Block>::size_type blockCount() const { return blocks.size(); }
			bool empty() const { return blocks.size() == 1 and blocks.front().free; }

		private:
			std::list<Block> blocks;
			/* Log aLog; */
	};  // class DynBlockAllocator
}  // namespace gz::vlk
