#include "renderer.hpp"

#include "vulkan_instance.hpp"

namespace gz::vlk {
    void Renderer::cleanup_(){
        vk.destroyCommandBuffers(commandBuffers);
        
        vk.destroyBuffer(indexBuffer, indexBufferMemory);
        vk.destroyBuffer(vertexBuffer, vertexBufferMemory);
    }



    void Renderer::createDescriptorSetLayoutSampler(uint32_t binding) {
        // combined image sampler
        std::vector<vk::DescriptorSetLayoutBinding> sampleFromRenderImageDSLB;
        sampleFromRenderImageDSLB.emplace_back(vk::DescriptorSetLayoutBinding {
            .binding = binding,
            .descriptorType = vk::DescriptorType::eCombinedImageSampler,
            .descriptorCount = 1,
            .stageFlags = vk::ShaderStageFlagBits::eFragment,
        });
        vk.createDescriptorSetLayout(sampleFromRenderImageDSLB, sampleFromRenderImageDSL);
    }


	void Renderer::createDescriptorSetSampler() {
        std::vector<vk::DescriptorSetLayout> layouts2(vk.getScImages().size(), sampleFromRenderImageDSL);
        vk.createDescriptorSets(layouts2, descriptorPool, sampleFromRenderImageDS);

        assert(renderImageViews.size() == vk.getScImages().size());
        assert(sampleFromRenderImageDS.size() == vk.getScImages().size());
        for (size_t i = 0; i < vk.getMaxFramesInFlight(); i++) {
            vk::DescriptorImageInfo imageI {
            	.sampler = imageSampler,
            	.imageView = renderImageViews[i],
            	.imageLayout = vk::ImageLayout::eShaderReadOnlyOptimal,
            };

            vk::WriteDescriptorSet descriptorW{
                .dstSet = sampleFromRenderImageDS[i],
                .dstBinding = 0,
                .dstArrayElement = 0,
                .descriptorCount = 1,
                .descriptorType = vk::DescriptorType::eCombinedImageSampler,
                .pImageInfo = &imageI,
            };
            // write 1, copy 0
            vk.getDevice().updateDescriptorSets(descriptorW, nullptr);
        } // for
        /* rLog.log0("createDescriptoSetSampler: Created descriptor layout and sets for sampling from renderImages"); */
    }
}
