#pragma once

/* #include <bits/ranges_base.h> */
#include <glm/glm.hpp>
#include <glm/gtx/hash.hpp>

#include <array>
#include <string>

// defined in vulkan/vulkan.hpp
namespace vk {
    struct VertexInputBindingDescription;
    struct VertexInputAttributeDescription;
}

namespace gz::vlk {
    template<typename T>
    concept VertexType = requires {
        requires std::same_as<decltype(T::getBindingDescription()), vk::VertexInputBindingDescription>;
        requires std::ranges::forward_range<decltype(T::getAttributeDescriptions())>;
        requires std::same_as<std::ranges::range_value_t<decltype(T::getAttributeDescriptions())>, vk::VertexInputAttributeDescription>;
    };

    template<typename T>
    concept GLM_vec2_or_3 = std::same_as<glm::vec3, T> or std::same_as<glm::vec2, T>;

    //
    // VERTEX
    //
    /**
     * @brief Base for 2D and 3D vertices with texture and/or color
     */
    template<GLM_vec2_or_3 PosVec>
    struct Vertex {
        PosVec pos = PosVec(0);
        glm::vec3 color = { 0, 0, 0 };
        glm::vec2 texCoord = { 0, 0 };
        std::string toString() const;
        bool operator==(const Vertex& other) const;
        static vk::VertexInputBindingDescription getBindingDescription();
        static std::array<vk::VertexInputAttributeDescription, 3> getAttributeDescriptions();
    }; // struct Vertex
    using Vertex3D = Vertex<glm::vec3>;
    using Vertex2D = Vertex<glm::vec2>;
} // namespace gz::vlk



//
// HASHES
//
namespace std {
    template<gz::vlk::GLM_vec2_or_3 PosVec>
    struct hash<gz::vlk::Vertex<PosVec>> {
        size_t operator()(gz::vlk::Vertex<PosVec> const& vertex) const {
            return ((hash<PosVec>()(vertex.pos)) ^ 
                    (hash<glm::vec3>()(vertex.color) << 1) >> 1 ) ^
                    (hash<glm::vec2>()(vertex.texCoord) << 1);
        }
    };
}
